"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stateFactory_1 = require("./stateFactory");
const staticGenerators_1 = require("./staticGenerators");
function moduleDecoratorFactory(moduleOptions) {
    return function (constructor) {
        const module = constructor;
        const stateFactory = () => stateFactory_1.stateFactory(module);
        if (!module.state) {
            module.state = (moduleOptions && moduleOptions.stateFactory) ? stateFactory : stateFactory();
        }
        if (!module.getters) {
            module.getters = {};
        }
        module.namespaced = moduleOptions && moduleOptions.namespaced;
        Object.getOwnPropertyNames(module.prototype).forEach((funcName) => {
            const descriptor = Object.getOwnPropertyDescriptor(module.prototype, funcName);
            if (descriptor.get) {
                module.getters[funcName] = (moduleState) => descriptor.get.call(moduleState);
            }
        });
        if (moduleOptions.dynamic) {
            const modOpt = moduleOptions;
            if (!modOpt.name) {
                throw new Error('Name of module not provided in decorator options');
            }
            let statics = {};
            // ===========  For statics ==============
            // ------ state -------
            staticGenerators_1.staticStateGenerator(module, modOpt, statics);
            // ------- getters -------
            if (module.getters) {
                staticGenerators_1.staticGetterGenerator(module, modOpt, statics);
            }
            // -------- mutations --------
            if (module.mutations) {
                staticGenerators_1.staticMutationGenerator(module, modOpt, statics);
            }
            // -------- actions ---------
            if (module.actions) {
                staticGenerators_1.staticActionGenerators(module, modOpt, statics);
            }
            modOpt.store.registerModule(modOpt.name, // TODO: Handle nested modules too in future
            module);
            Object.defineProperty(constructor, '_statics', {
                value: statics
            });
            return constructor;
        }
    };
}
function Module(modOrOpt) {
    if (typeof modOrOpt === 'function') {
        /*
         * @Module decorator called without options (directly on the class definition)
         */
        moduleDecoratorFactory({})(modOrOpt);
    }
    else {
        /*
         * @Module({...}) decorator called with options
         */
        return moduleDecoratorFactory(modOrOpt);
    }
}
exports.Module = Module;
//# sourceMappingURL=index.js.map